%{

#include "flex-actions.h"

%}

/* Obliga a crear una regla explícita para manejar lexemas desconocidos. */
%option nodefault

/* Escanea un único archivo y se detiene. */
%option noyywrap

/* Dispone una variable global "yylineno" con el número de línea actual. */
%option yylineno

/* Patrones reutilizables. */
crlf			\r\n
digit			[0-9]
char            [a-zA-Z]
decimal         [.]
endline			\n
whitespace		[ \f\n\r\t\v]

%%

"+"											{ return ADD; }
"-"											{ return SUB; }
"*"											{ return MUL; }
"/"											{ return DIV; }
"^"											{ return POW; }

"fun"                                       { return FUNCTION; }
"function"                                  { return FUNCTION; }

"if"                                        { return IF; }
"else"                                      { return ELSE; }
"for"                                       { return FOR; }
"print"                                     { return PRINT; }
";"                                         { return SEMICOLON; }

"o"                                         { return COMPOSITE; }
"'"+                                        { return DERIVATIVE; }
"derivative"                                { return DERIVATIVE; }

"integrate"                                 { return INTEGRATE; }
"int"                                       { return INTEGRATE; }

"between"                                   { return BETWEEN; }
"and"                                       { return AND; }
"~"                                         { return AND; }

"evaluate"                                  { return EVALUATE; }
"in"                                        { return IN; }

"{"                                         { return OPEN_BRACES; }
"}"                                         { return CLOSE_BRACES; } 

"{"                                         { return OPEN_BRACKETS; }
"}"                                         { return CLOSE_BRACKETS; } 

"("											{ return OPEN_PARENTHESIS; }
")"											{ return CLOSE_PARENTHESIS; }

":"                                         { return COLON; } 
","                                         { return COMMA; } 
">"                                         { return GREATER; } 
">="                                        { return GREATER_EQUAL; } 
"<"                                         { return LESSER; } 
"<="                                        { return LESSER_EQUAL; } 
"="                                         { return ASSIGN; } 
"=="                                        { return EQUAL; } 

"+="                                        { return ADD_ASSIGN; } 
"++"                                        { return INCREMENT; } 
"-="                                        { return SUB_ASSIGN; } 
"--"                                        { return DECREMENT; } 
"\""                                        { return QUOTE; }

{digit}+									{ return IntegerPatternAction(yytext); }

{digit}+{decimal}{digit}+                   { return DoublePatternAction(yytext); }

{char}+                                     { return StringPatternAction(yytext); }

{whitespace}								{ IgnoredPatternAction(yytext); }

.											{ return UnknownPatternAction(yytext); }

%%
